from kivy.uix.widget import Widget
from kivy.resources import resource_find
from kivy.graphics.transformation import Matrix
from kivy.graphics.opengl import *
from kivy.graphics import *


class Scene(Widget):
    min_pitch = -20
    max_pitch = 90

    yaw_speed = 15
    pitch_speed = 15

    x_sensitivity = .9
    y_sensitivity = .7

    warp_speed = 0
    warp_accel = .002
    warp_decel = .1

    def __init__(self, app):
        self.app = app
        self.fov = 90

        self.canvas = RenderContext(compute_normal_matrix=True)
        self.canvas.shader.source = resource_find('shaders/simple.glsl')
        self.canvas['projection_mat'] = Matrix()
        self.canvas['warp'] = 0.0

        Widget.__init__(self)
        with self.canvas:
            self.cb = Callback(self.setup_gl_context)
            PushMatrix()

            self.cam_offset = Translate(0, 0, -6)
            self.cam_pitch = Rotate(0, 1, 0, 0)
            self.cam_yaw = Rotate(0, 0, 1, 0)
            self.cam_pos = Translate(0, 0, -5)

        with self.canvas.after:
            PopMatrix()
            self.cb = Callback(self.reset_gl_context)

        self.target_yaw = 0
        self.target_pitch = 0

        self.warping = False

    def setup_gl_context(self, *args):
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_CULL_FACE)

    def reset_gl_context(self, *args):
        glDisable(GL_DEPTH_TEST)
        glDisable(GL_CULL_FACE)

    def on_size(self, *args):
        self.reset_matrix()

    def set_fov(self, new_fov):
        self.fov = new_fov
        self.reset_matrix()

    def reset_matrix(self):
        self.canvas['projection_mat'].perspective(self.fov, self.width / float(self.height),
                                                  .1, 400)

    def on_touch_move(self, touch):
        self.target_yaw += touch.dx * self.x_sensitivity
        self.target_pitch = max(self.min_pitch,
                                min(self.max_pitch,
                                    self.target_pitch - touch.dy * self.y_sensitivity))

    def set_cam_pos(self, x, y, z):
        self.cam_pos.xyz = -x, -y, -z

    def update_camera(self, dt):
        yaw_diff = ((self.target_yaw - self.cam_yaw.angle) + 180) % 360 - 180
        self.cam_yaw.angle += yaw_diff * self.yaw_speed * dt

        pitch_diff = ((self.target_pitch - self.cam_pitch.angle) + 180) % 360 - 180
        self.cam_pitch.angle += pitch_diff * self.pitch_speed * dt

    def update(self, dt):
        if self.warping:
            self.warp_speed += self.warp_accel * dt
            self.canvas['warp'] += self.warp_speed
        elif self.canvas['warp'] > 0:
            self.warp_speed -= self.warp_decel * dt
            self.canvas['warp'] += self.warp_speed
            if self.canvas['warp'] < 0:
                self.warp_speed = 0
                self.canvas['warp'] = 0.0
